@@ -4,23 +4,26 @@ import logging
 import traceback
 from asyncio import AbstractEventLoop, Task, CancelledError
 from logging import Logger
-from typing import Optional, Type, AsyncIterator, Dict, Any, Tuple, Union, Callable, List, Coroutine
+from typing import Optional, Type, Dict, Any, Union, Callable, List, Coroutine, AsyncIterator
 
-from httpx import Proxy
+import httpx
 from pyee.asyncio import AsyncIOEventEmitter
 from pyee.base import Handler
 
-from TikTokLive.client.errors import AlreadyConnectedError, UserOfflineError, InitialCursorMissingError, \
-    WebsocketURLMissingError, UserNotFoundError
+from TikTokLive.client.errors import AlreadyConnectedError, UserOfflineError, UserNotFoundError
 from TikTokLive.client.logger import TikTokLiveLogHandler, LogLevel
+from TikTokLive.client.web.routes.fetch_signed_websocket import WebcastPlatform
+from TikTokLive.client.web.routes.fetch_user_unique_id import FailedResolveUserId
 from TikTokLive.client.web.web_client import TikTokWebClient
 from TikTokLive.client.web.web_settings import WebDefaults
 from TikTokLive.client.ws.ws_client import WebcastWSClient
-from TikTokLive.events import Event, EventHandler
-from TikTokLive.events.custom_events import WebsocketResponseEvent, ConnectEvent, FollowEvent, ShareEvent, LiveEndEvent, \
-    DisconnectEvent, LivePauseEvent, LiveUnpauseEvent, UnknownEvent, CustomEvent
-from TikTokLive.events.proto_events import EVENT_MAPPINGS, ProtoEvent, ControlEvent
-from TikTokLive.proto import WebcastResponse, WebcastResponseMessage, ControlAction
+from TikTokLive.client.ws.ws_connect import WebcastProxy
+from TikTokLive.events import Event, EventHandler, ControlEvent
+from TikTokLive.events.custom_events import WebsocketResponseEvent, FollowEvent, ShareEvent, LiveEndEvent, \
+    DisconnectEvent, LivePauseEvent, LiveUnpauseEvent, UnknownEvent, CustomEvent, ConnectEvent, SuperFanEvent
+from TikTokLive.events.proto_events import EVENT_MAPPINGS, ProtoEvent, BarrageEvent
+from TikTokLive.proto import ProtoMessageFetchResult, ProtoMessageFetchResultBaseProtoMessage
+from TikTokLive.proto.custom_proto import ControlAction
 
 
 class TikTokLiveClient(AsyncIOEventEmitter):
@@ -31,11 +34,19 @@ class TikTokLiveClient(AsyncIOEventEmitter):
 
     def __init__(
             self,
-            unique_id: str,
-            web_proxy: Optional[Proxy] = None,
-            ws_proxy: Optional[Proxy] = None,
+            # User to connect to
+            unique_id: str | int,
+            platform: WebcastPlatform = WebcastPlatform.WEB,
+
+            # Proxies
+            web_proxy: Optional[httpx.Proxy] = None,
+            ws_proxy: Optional[WebcastProxy] = None,
+
+            # Client kwargs
             web_kwargs: Optional[dict] = None,
-            ws_kwargs: Optional[dict] = None
+            ws_kwargs: Optional[dict] = None,
+
+            is_user_id: Optional[bool] = False
     ):
         """
         Instantiate the TikTokLiveClient client
@@ -45,6 +56,7 @@ class TikTokLiveClient(AsyncIOEventEmitter):
         :param ws_proxy: An optional proxy used for the WebSocket connection
         :param web_kwargs: Optional arguments used by the HTTP client
         :param ws_kwargs: Optional arguments used by the WebSocket client
+        :param is_user_id: Optional argument to resolve userid to unique_id
 
         """
 
@@ -52,14 +64,17 @@ class TikTokLiveClient(AsyncIOEventEmitter):
 
         self._ws: WebcastWSClient = WebcastWSClient(
             ws_kwargs=ws_kwargs or {},
-            proxy=ws_proxy
+            ws_proxy=ws_proxy
         )
 
         self._web: TikTokWebClient = TikTokWebClient(
-            httpx_kwargs=web_kwargs or {},
-            proxy=web_proxy
+            web_proxy=web_proxy or (web_kwargs or {}).pop("web_proxy", None),
+            **(web_kwargs or {})
         )
 
+        self._web.params['referer'] = f"https://www.tiktok.com/@{unique_id}/live"
+        self._web.params['root_referer'] = f"https://www.tiktok.com/@{unique_id}/live"
+
         self._logger: Logger = TikTokLiveLogHandler.get_logger(
             level=LogLevel.ERROR
         )
@@ -68,6 +83,8 @@ class TikTokLiveClient(AsyncIOEventEmitter):
         self.ignore_broken_payload: bool = False
 
         # Properties
+        self._is_user_id: bool = is_user_id
+        self._ws_platform: WebcastPlatform = platform
         self._unique_id: str = self.parse_unique_id(unique_id)
         self._room_id: Optional[int] = None
         self._room_info: Optional[Dict[str, Any]] = None
@@ -94,10 +111,11 @@ class TikTokLiveClient(AsyncIOEventEmitter):
             self,
             *,
             process_connect_events: bool = True,
+            compress_ws_events: bool = True,
             fetch_room_info: bool = False,
             fetch_gift_info: bool = False,
             fetch_live_check: bool = True,
-            room_id: Optional[int] = None
+            room_id: Optional[int] = None,
     ) -> Task:
         """
         Create a non-blocking connection to TikTok LIVE and return the task
@@ -108,6 +126,7 @@ class TikTokLiveClient(AsyncIOEventEmitter):
         :param fetch_live_check: Whether to check if the user is live (you almost ALWAYS want this enabled)
         :param room_id: An override to the room ID to connect directly to the livestream and skip scraping the live.
                         Useful when trying to scale, as scraping the HTML can result in TikTok blocks.
+        :param compress_ws_events: Whether to compress the WebSocket events using gzip compression (you should probably have this on)
         :return: Task containing the heartbeat of the client
 
         """
@@ -115,21 +134,25 @@ class TikTokLiveClient(AsyncIOEventEmitter):
         if self._ws.connected:
             raise AlreadyConnectedError("You can only make one connection per client!")
 
+        self._unique_id = await self._resolve_user_id(self._unique_id)
+
         # <Required> Fetch room ID
         try:
-            self._room_id: int = room_id or await self._web.fetch_room_id_from_html(self._unique_id)
-            self._web.params["room_id"] = str(self._room_id) or None
+            self._room_id: int = int(room_id or await self._web.fetch_room_id_from_html(self._unique_id))
         except Exception as base_ex:
 
             if isinstance(base_ex, UserOfflineError) or isinstance(base_ex, UserNotFoundError):
                 raise base_ex
 
             try:
-                self._logger.error("Failed to parse room ID from HTML. Using API fallback.")
-                self._room_id: int = await self._web.fetch_room_id_from_api(self.unique_id)
+                self._logger.debug("Failed to parse room ID from HTML. Using API fallback.")
+                self._room_id: int = int(await self._web.fetch_room_id_from_api(self.unique_id))
             except Exception as super_ex:
                 raise super_ex from base_ex
 
+        # Gram Room ID
+        self._web.params["room_id"] = str(self._room_id) or None
+
         # <Optional> Fetch live status
         if fetch_live_check and not await self._web.fetch_is_live(room_id=self._room_id):
             raise UserOfflineError()
@@ -143,21 +166,17 @@ class TikTokLiveClient(AsyncIOEventEmitter):
             self._gift_info = await self._web.fetch_gift_list()
 
         # <Required> Fetch the first response
-        webcast_response: WebcastResponse = await self._web.fetch_sign_fetch()
-
-        # <Optional> Disregard initial events
-        webcast_response.messages = webcast_response.messages if process_connect_events else []
-
-        # Handle detection & invalid payloads
-        if not webcast_response.cursor:
-            raise InitialCursorMissingError("Missing cursor in initial fetch response.")
-        if not webcast_response.push_server:
-            raise WebsocketURLMissingError("No websocket URL received from TikTok.")
-        if not webcast_response.route_params_map:
-            raise WebsocketURLMissingError("Websocket parameters missing.")
+        initial_webcast_response: ProtoMessageFetchResult = await self._web.fetch_signed_websocket(self._ws_platform)
 
         # Start the websocket connection & return it
-        self._event_loop_task = self._asyncio_loop.create_task(self._client_loop(webcast_response))
+        self._event_loop_task = self._asyncio_loop.create_task(
+            self._ws_client_loop(
+                initial_webcast_response=initial_webcast_response,
+                process_connect_events=process_connect_events,
+                compress_ws_events=compress_ws_events
+            )
+        )
+
         return self._event_loop_task
 
     async def connect(
@@ -206,103 +225,48 @@ class TikTokLiveClient(AsyncIOEventEmitter):
 
         return self._asyncio_loop.run_until_complete(self.connect(**kwargs))
 
-    async def disconnect(self) -> None:
+    async def disconnect(self, close_client: bool = False) -> None:
         """
-        Disconnect the client from the websocket
+        Disconnect the client from the websocket.
 
+        :param close_client: Whether to also close the HTTP client if you don't intend to reuse it
         :return: None
 
         """
 
-        # Wait gracefully for things to finish
+        # Disconnect the WebSocket
         await self._ws.disconnect()
-        await self._event_loop_task
 
-        # If recording, stop
-        if self._web.fetch_video.is_recording:
-            self._web.fetch_video.stop()
+        # Wait for the event loop task to finish
+        if self._event_loop_task is not None:
+            try:
+                await self._event_loop_task
+            except Exception:
+                self._logger.debug("an exception in event loop is ignored", exc_info=True)
+            self._event_loop_task = None
+
+        # If recording, stop it
+        if self._web.fetch_video_data.is_recording:
+            self._web.fetch_video_data.stop()
+
+        # Close the client (if discarding)
+        if close_client:
+            await self.close()
 
         # Reset state vars
         self._room_id = None
         self._room_info = None
         self._gift_info = None
-        self._event_loop_task = None
 
-    async def _client_loop(self, initial_response: WebcastResponse) -> None:
+    async def close(self) -> None:
         """
-        Run the main client loop to handle events
+        Discards the async sessions if you don't intend to use the client again
 
-        :param initial_response: The WebcastResponse retrieved from the sign server with connection info
         :return: None
 
         """
 
-        async for event in self._ws_loop(initial_response):
-
-            if event is None:
-                continue
-
-            self._logger.debug(f"Received Event '{event.type}'.")
-            self.emit(event.type, event)
-
-        # Disconnecting
-        ev: DisconnectEvent = DisconnectEvent()
-        self.emit(ev.type, ev)
-
-    async def _ws_loop(self, initial_response: WebcastResponse) -> AsyncIterator[Optional[Event]]:
-        """
-        Run the websocket loop to handle incoming WS messages
-
-        :param initial_response: The WebcastResponse retrieved from the sign server with connection info
-        :return: None
-
-        """
-
-        first_event: bool = True
-
-        # Handle websocket connection
-        async for response_message in self._ws.connect(*self._build_connect_info(initial_response)):
-
-            if first_event:
-                first_event = False
-
-                # Send a connection event
-                yield ConnectEvent(unique_id=self._unique_id, room_id=self._room_id)
-
-                # Handle initial messages
-                for webcast_message in initial_response.messages:
-                    for event in await self._parse_webcast_response(webcast_message):
-                        yield event
-
-            for event in await self._parse_webcast_response(response_message):
-                yield event
-
-    def _build_connect_info(self, initial_response: WebcastResponse) -> Tuple[str, dict]:
-        """
-        Create connection info for starting the connection
-
-        :param initial_response: The WebcastResponse retrieved from the sign server with connection info
-        :return: None
-
-        """
-
-        uri_params: dict = {
-            "room_id": self._room_id,
-            **WebDefaults.client_ws_params,
-            **initial_response.route_params_map
-        }
-
-        connect_uri: str = (
-                initial_response.push_server
-                + "?"
-                + '&'.join(f"{key}={value}" for key, value in uri_params.items())
-        )
-
-        connect_headers: dict = {
-            "Cookie": " ".join(f"{k}={v};" for k, v in self._web.cookies.items())
-        }
-
-        return connect_uri, connect_headers
+        await self._web.close()
 
     def on(self, event: Type[Event], f: Optional[EventHandler] = None) -> Union[Handler, Callable[[Handler], Handler]]:
         """
@@ -341,43 +305,101 @@ class TikTokLiveClient(AsyncIOEventEmitter):
 
         return event.__name__ in self._events
 
-    async def _parse_webcast_response(self, response: Optional[WebcastResponseMessage]) -> List[Event]:
+    async def _ws_client_loop(
+            self,
+            initial_webcast_response: ProtoMessageFetchResult,
+            process_connect_events: bool,
+            compress_ws_events: bool
+    ) -> None:
+        """
+        Run the websocket loop to handle incoming WS events
+
+        :param initial_webcast_response: The ProtoMessageFetchResult (as bytes) retrieved from the sign server with connection info
+        :param process_connect_events: Whether to process initial events sent on room join
+        :param compress_ws_events: Whether to compress the WebSocket events using gzip compression
+        :return: None
+
+        """
+
+        # Handle websocket connection
+        async for webcast_response in self._ws.connect(
+                initial_webcast_response=initial_webcast_response,
+                process_connect_events=process_connect_events,
+                compress_ws_events=compress_ws_events,
+                cookies=self._web.cookies,
+                room_id=self._room_id,
+                user_agent=self._web.headers['User-Agent']
+        ):
+
+            # Iterate over the events extracted
+            async for event in self._parse_webcast_response(webcast_response):
+                self._logger.debug(f"Received Event '{event.type}' [{event.size} bytes]")
+                self.emit(event.type, event)
+
+        # Send the Disconnect event when we disconnect
+        ev: DisconnectEvent = DisconnectEvent()
+        self.emit(ev.type, ev)
+
+    async def _parse_webcast_response(self, webcast_response: ProtoMessageFetchResult) -> AsyncIterator[Event]:
         """
         Parse incoming webcast responses into events that can be emitted
 
-        :param response: The WebcastResponseMessage protobuf message
+        :param webcast_response: The ProtoMessageFetchResult protobuf message
+        :return: A list of events that can be gleamed from this event
+
+        """
+
+        # The first event means we connected
+        if webcast_response.is_first:
+            yield ConnectEvent(unique_id=self._unique_id, room_id=self._room_id)
+
+        # Yield events
+        for message in webcast_response.messages:
+            for event in await self._parse_webcast_response_message(webcast_response_message=message):
+                if event is not None:
+                    yield event
+
+    async def _parse_webcast_response_message(
+            self,
+            webcast_response_message: Optional[ProtoMessageFetchResultBaseProtoMessage]
+    ) -> List[Event]:
+        """
+        Parse incoming webcast responses into events that can be emitted
+
+        :param webcast_response_message: The ProtoMessageFetchResultMessage protobuf message
         :return: A list of events that can be gleamed from this event
 
         """
 
         # Invalid response handler
-        if response is None:
-            self._logger.warning("Received a null WebcastResponseMessage from the Webcast server.")
+        if webcast_response_message is None:
+            self._logger.warning("Received a null ProtoMessageFetchResultMessage from the Webcast server.")
             return []
 
         # Get the proto mapping for proto-events
-        event_type: Optional[Type[ProtoEvent]] = EVENT_MAPPINGS.get(response.method)
-        response_event: Event = WebsocketResponseEvent().from_dict(response.to_dict())
+        event_type: Optional[Type[ProtoEvent]] = EVENT_MAPPINGS.get(webcast_response_message.method)
+        response_event: Event = WebsocketResponseEvent().from_dict(webcast_response_message.to_dict())
 
         # If the event is not tracked, return
         if event_type is None:
-            return [response_event, UnknownEvent().from_dict(response.to_dict())]
+            return [response_event, UnknownEvent().from_dict(webcast_response_message.to_dict())]
 
         # Get the underlying events
         try:
-            proto_event: ProtoEvent = event_type().parse(response.payload)
+            proto_event: ProtoEvent = event_type().parse(webcast_response_message.payload)
         except Exception:
             if not self.ignore_broken_payload:
-                self._logger.error(traceback.format_exc() + "\nBroken Payload:\n" + str(response.payload))
+                self._logger.error(
+                    traceback.format_exc() + "\nBroken Payload:\n" + str(webcast_response_message.payload))
             return [response_event]
 
         parsed_events: List[Event] = [response_event, proto_event]
-        custom_event: Optional[Event] = await self.handle_custom_event(response, proto_event)
+        custom_event: Optional[Event] = await self.handle_custom_event(webcast_response_message, proto_event)
 
         # Add the custom event IF not null
         return [custom_event, *parsed_events] if custom_event else parsed_events
 
-    async def is_live(self, unique_id: Optional[str] = None) -> bool:
+    async def is_live(self, unique_id: Optional[str | int] = None) -> bool:
         """
         Check if the client is currently live on TikTok
 
@@ -386,43 +408,74 @@ class TikTokLiveClient(AsyncIOEventEmitter):
 
         """
 
+        self._unique_id = unique_id = await self._resolve_user_id(unique_id or self.unique_id)
+
         return await self._web.fetch_is_live(unique_id=unique_id or self.unique_id)
 
-    async def handle_custom_event(self, response: WebcastResponseMessage, event: ProtoEvent) -> Optional[CustomEvent]:
+    async def handle_custom_event(self, response: ProtoMessageFetchResultBaseProtoMessage, event: ProtoEvent) -> \
+            Optional[CustomEvent]:
         """
         Extract CustomEvent events from existing ProtoEvent events
 
-        :param response: The WebcastResponseMessage to parse for the custom event
+        :param response: The ProtoMessageFetchResultMessage to parse for the custom event
         :param event: The ProtoEvent to parse for the custom event
         :return: The event, if one exists
 
         """
 
+        if isinstance(event, BarrageEvent) and "ttlive_superFan" in event.content.key:
+            return SuperFanEvent().parse(response.payload)
+
         # LiveEndEvent, LivePauseEvent, LiveUnpauseEvent
         if isinstance(event, ControlEvent):
-            if event.action == ControlAction.STREAM_ENDED:
-
-                # If the stream is over, disconnect the client
-                self.disconnect()
-
+            if event.action in {
+                ControlAction.CONTROL_ACTION_STREAM_ENDED,
+                ControlAction.CONTROL_ACTION_STREAM_SUSPENDED
+            }:
+                # If the stream is over, disconnect the client. Can't await due to circular dependency.
+                self._asyncio_loop.create_task(self.disconnect())
                 return LiveEndEvent().parse(response.payload)
-            elif event.action == ControlAction.STREAM_PAUSED:
+            elif event.action == ControlAction.CONTROL_ACTION_STREAM_PAUSED:
                 return LivePauseEvent().parse(response.payload)
-            elif event.action == ControlAction.STREAM_PAUSED:
+            elif event.action == ControlAction.CONTROL_ACTION_STREAM_PAUSED:
                 return LiveUnpauseEvent().parse(response.payload)
             return None
 
         # FollowEvent
-        if "follow" in event.common.display_text.key:
+        if "follow" in event.base_message.display_text.key:
             return FollowEvent().parse(response.payload)
 
         # ShareEvent
-        if "share" in event.common.display_text.key:
+        if "share" in event.base_message.display_text.key:
             return ShareEvent().parse(response.payload)
 
         # Not a custom event
         return None
 
+    async def _resolve_user_id(self, unique_id: str | int) -> str:
+        """Resolve a unique_id and return the resolved value"""
+        parsed_id = self.parse_unique_id(unique_id)
+        if parsed_id.isdigit() and self._is_user_id:
+            resolved_id = await self._web.fetch_user_unique_id(int(parsed_id))
+            if not resolved_id:
+                raise FailedResolveUserId(f"Resolved ID is invalid: {resolved_id}")
+            return resolved_id
+        return parsed_id
+
+    async def send_room_chat(
+            self,
+            content: str
+    ) -> Any:
+        """
+        Send a chat message to the room
+
+        :param content: The content of the message
+        :return: The response from TikTok
+
+        """
+
+        return await self._web.send_room_chat(content=content, room_id=self._room_id)
+
     @property
     def unique_id(self) -> str:
         """
@@ -433,7 +486,7 @@ class TikTokLiveClient(AsyncIOEventEmitter):
         return self._unique_id
 
     @property
-    def room_id(self) -> Optional[str]:
+    def room_id(self) -> Optional[int]:
         """
         The room ID the user is currently connected to
 
